"""A py.test plugin to check ``gettext`` ``po`` & ``mo`` files."""


def pytest_addoption(parser):
    group = parser.getgroup("general")
    group.addoption(
        "--translations",
        action="store_true",
        help="perform some checks on .mo and .po files",
    )


def pytest_configure(config):
    config.addinivalue_line("markers", "translations: translation tests")


def pytest_collect_file(file_path, parent):
    from .mo_files import MoFile
    from .po_files import PoFile

    config = parent.config
    if config.option.translations:
        if file_path.suffix == ".mo":
            return MoFile.from_parent(
                path=file_path,
                parent=parent,
            )
        elif file_path.suffix == ".po":
            return PoFile.from_parent(path=file_path, parent=parent)



import os
import shutil
import tempfile

from pytest import File, Item

from pytest_translations.config import MARKER_NAME
from pytest_translations.utils import (
    TranslationException,
    msgfmt,
    open_mo_file,
    open_po_file,
)


class MoFile(File):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if hasattr(self, "add_marker"):
            self.add_marker(MARKER_NAME)
        else:
            self.keywords[MARKER_NAME] = True

    def collect(self):
        yield MoItem.from_parent(
            name=self.name,
            parent=self,
        )


class MoItem(Item):
    def __init__(self, name, parent):
        super().__init__(name, parent)
        self.add_marker(MARKER_NAME)

    def runtest(self):
        po_path, _ = os.path.splitext(str(self.path))
        po_path += ".po"

        if not os.path.exists(po_path):
            raise TranslationException("corresponding .po file does not exist", [])

        po_file = open_po_file(po_path)

        temp_dir = tempfile.mkdtemp()

        try:
            test_file = os.path.join(temp_dir, "test.mo")
            po_file.save_as_mofile(test_file)

            original_parsed = open_mo_file(self.path)
            test_parsed = open_mo_file(test_file)

            if len(original_parsed) != len(test_parsed):
                raise TranslationException("mo-file length does not match po.", [])

            diff = []

            for lhs, rhs in zip(original_parsed, test_parsed):
                if not (
                    lhs.msgid == rhs.msgid
                    and lhs.msgid_plural == rhs.msgid_plural
                    and lhs.msgstr == rhs.msgstr
                    and lhs.msgstr_plural == rhs.msgstr_plural
                ):
                    diff.append((lhs, rhs))

            if diff:
                raise TranslationException(
                    "mo-file content does not match po.",
                    diff,
                )

        finally:
            shutil.rmtree(temp_dir)

    def repr_failure(self, excinfo):
        if isinstance(excinfo.value, TranslationException):
            msg, diff = excinfo.value.args

            msg += "\n" + "\n".join(
                "{0} -> {1}".format(
                    msgfmt(lhs),
                    msgfmt(rhs),
                )
                for lhs, rhs in diff
            )

            return msg

        else:
            return super().repr_failure(excinfo)

    def reportinfo(self):
        return (self.path, -1, "mo-test")



import os
import re

from pytest import Item, skip

from pytest_translations.config import MARKER_NAME
from pytest_translations.utils import TranslationException

try:
    import enchant
    from enchant.tokenize import EmailFilter, HTMLChunker, URLFilter, get_tokenizer
except ImportError:
    enchant = None


def is_enchant_installed():
    return not (enchant is None)


if enchant:
    supported_languages = [name.split("_")[0] for name, _ in enchant.list_dicts()]
else:
    supported_languages = []


def word_lists():
    PWL = os.getenv("PYTEST_TRANSLATIONS_PRIVATE_WORD_LIST")

    if enchant and PWL and os.path.exists(PWL):
        return {
            fn: os.path.join(PWL, fn) for fn in os.listdir(PWL) if not os.path.isdir(fn)
        }

    else:
        return {}


class PoSpellCheckingItem(Item):
    def __init__(self, line, language, name, parent):
        self.line = line
        self.language = language

        wl = word_lists()

        private_word_list = wl.get(language, None)

        if enchant and language in supported_languages:
            self.lang_dict = enchant.DictWithPWL(language, pwl=private_word_list)
        else:
            self.lang_dict = None

        super().__init__(name, parent)
        self.add_marker(MARKER_NAME)

    def runtest(self):
        if not enchant:
            skip("enchant is not installed")

        if not self.language:
            skip("no language defined in PO file")

        if self.language not in supported_languages:
            skip("aspell dictionary for language {} not found.".format(self.language))

        text = self.line.msgstr

        # remove python format placeholders, old and new,
        # where it's a problem.
        # 1. replace everything between curly braces
        text = re.sub("{.*?}", "", text)
        # 2. remove everything between %( and )
        text = re.sub(r"\%\(.*?\)", "", text)
        # 3. remove &shy; html entity
        text = text.replace("&shy;", "")

        tokenizer = get_tokenizer(
            chunkers=[
                HTMLChunker,
            ],
            filters=(
                EmailFilter,
                URLFilter,
            ),
        )
        tokens = tokenizer(text)

        errors = [t for t, _ in tokens if not self.lang_dict.check(t)]

        if errors:
            raise TranslationException(
                "Spell checking failed: {}".format(", ".join(errors)),
                self.line.msgstr,
                [(t, self.lang_dict.suggest(t)) for t in errors],
            )

    def repr_failure(self, excinfo):
        if isinstance(excinfo.value, TranslationException):
            msg, line, wrong = excinfo.value.args
            lines = [msg, str(self.path), 'msgstr "%s"' % line]
            for i in wrong:
                lines.append("%s: %s" % i)

            return "\n".join(lines)

        else:
            return super().repr_failure(excinfo)

    def reportinfo(self):
        return (self.path, -1, "po-spelling")
